// --------------------------------------------
//  File name:  ElectronicKeyer v4
//  Description: パドルを接続しエレキーを構成する
//  SWで短点長を50-240ms設定する。
//  Notes: 1MHz内部クロック
//      keyDOT  !RA5   keyDASH  !RA4
//      sw_P    !RA3   sw_M     !RA2
//      CW 出力   RA1   Tone出力   RA0
//    Language: MPLABX  XC8 v2.2  Target: PIC16F18313
//    20210102 Ver 2.0 縦振キー機能追加
//    20210105 Ver 3.0 EEPROM追加
//    20210105 Ver 4.0 文字間スペース機能追加
//    20210108 Ver 4.1 待機ループ内で電鍵CK
// ----------------------------------------------

#include <xc.h>
#define _XTAL_FREQ  1000000

#define keyDOT      !RA5
#define keyDASH     !RA4
#define sw_M        !RA3
#define sw_P        !RA2
#define lenSp       lenDOT
#define CWout       LATA1
#define ToneOn()    TRISA0=0
#define ToneOff()   TRISA0=1


#pragma config FEXTOSC = OFF,RSTOSC = HFINT1  // HFINTOSC (1MHz)
#pragma config CLKOUTEN = OFF,CSWEN = OFF,FCMEN = OFF
#pragma config MCLRE = OFF,PWRTE = OFF,WDTE = OFF,LPBOREN = OFF
#pragma config BOREN = OFF,BORV = LOW,PPS1WAY = OFF,STVREN = ON
#pragma config DEBUG = OFF
#pragma config WRT = OFF,LVP = OFF,CP = OFF,CPD = OFF

// 符号送信速度　---------------------------------------
// 段位      ３段 ２段 初段 １級 ２級 ３級
// 字/分     120 110  90  60  45	 25
// 短点長 ms  50  55  67 100 133 240
const int speed[] = {50,55,67,100,133,240};

//------ プログラム直後のEEPROMの初期データ --------
__EEPROM_DATA(250,3,0,0,0,0,0,0); //0番地:Freq, 1番地:idxSpd

/* ++++++++  共通変数 ++++++++ */
signed char idxSpd = 3;
int lenDOT;
char flagDASH = 0;
char flagDOT  = 0;
char reqCharSp = 0;

/* ++++++++  プロトタイプ ++++++++ */
void SigDOT(void);
void delayDOT (int);
void SigDASH(void);
void delayDASH (int);
void CharSP(void);
void SpeedUp(void);
void SpeedDown(void);
void beep(void);
void StrightKey(void);
void FreqUp(void);
void FreqDown(void);

/* ++++++++  Main ++++++++ */
void main(void)
{
    LATA   = 0b000000;
    TRISA  = 0b111101;          // RA1 は出力
    ANSELA = 0b000000;          // 全てデジタル
    WPUA   = 0b111100;          // RA2-RA5はプルアップ

    // Timer0 設定　
    T0CON1 = 0b01000000;    // Fosc/4 Sync PreS 1/1
    T0CON0 = 0b10000000;    // T0on 8bit Post 1/1
    RA0PPS = 0b11100;       // TMR0をRA0に出力
//    TMR0H  = 250;           // 500Hz:250 800Hz:156 1kHz:125
    TMR0H = eeprom_read(0);     // EEPROM 0番地 読み込み 周波数
    idxSpd = eeprom_read(1);    // EEPROM 1番地 読み込み　速度

    if(sw_P) StrightKey();  // 縦振キー機能

    // Timer2 設定 1.5ms x PR2 で、CountUpする
    T2CON = 0b00101111;         // Pre 1:64; Post 1:6; TMR2ON

    lenDOT = speed[idxSpd];     // 速度値を取得

    while (1)
    {
        if(keyDOT) flagDOT = 1;   	// 短点が押されたなら記録
        if(flagDOT) SigDOT();     	// 短点が記録なら短点処理
        if(keyDASH) flagDASH = 1;  	// 長点が押されたなら記録
        if(flagDASH) SigDASH();  	// 長点が記録なら長点処理
        if(!flagDOT && !flagDASH) CharSP();    // Keyなしなら文字間スペース
        if(sw_P) SpeedUp();      // + SWなら速度を１段階早くする
        if(sw_M) SpeedDown();     // - SWなら速度を１段階遅くする
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 短点とマーク間スペースを送信
//++++++++++++++++++++++++++++++++++++++++++++++
void SigDOT(void){
    while(TMR2IF == 0){         // 文字間スペースを確保
        __delay_us(952);            // 1ms 待ち
        if(keyDASH) flagDASH = 1;   // 長点が押されたなら長点記憶
    }
    CWout = 1;                  // CW出力　On
    ToneOn();                   // Tone出力　On
    delayDOT(lenDOT);           // 短点の長さだけ遅延
    CWout = 0;                  // CW出力　Off
    ToneOff();                  // Tone出力　Off
    delayDOT(lenSp);            // 符号間スペース
    reqCharSp = 1;              // 文字間スペース要求をセット
    flagDOT = keyDOT;           // 短点Keyで、フラグ On/Off
}

void delayDOT(int len){
    while(len--){                   // 指定の長さだけ繰り返す
        __delay_us(952);            // 1ms 待ち
        if(keyDASH) flagDASH = 1;   // 長点が押されたなら長点記憶
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 長点とマーク間スペースを送信 長点時間は短点の３倍
//++++++++++++++++++++++++++++++++++++++++++++++
void SigDASH(void){
    while(TMR2IF == 0){         // 文字間スペースを確保
        __delay_us(952);         // 1ms 待ち
        if(keyDOT) flagDOT = 1;  // 短点が押されたなら短点記憶
    }
    CWout = 1;                  // CW出力　On
    ToneOn();                   // Tone出力　On
    delayDASH(lenDOT * 3);      // 長点の長さだけ遅延
    CWout = 0;                  // CW出力　Off
    ToneOff();                  // Tone出力　Off
    delayDASH(lenSp);           // 符号間スペース
    reqCharSp = 1;              // 文字間スペース要求をセット
    flagDASH = keyDASH;         // 長点Keyで、フラグ On/Off
}

void delayDASH(int len){
    while(len--){                // 指定の長さだけ繰り返す
        __delay_us(952);         // 1ms 待ち
        if(keyDOT) flagDOT = 1;  // 短点が押されたなら短点記憶
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 文字間スペースを送信　 Keyが押されていい時に呼ばれる
// 既に時間スペース挿入後はすぐに戻る
// Timer2のフラグは、1.5msxPR時間経過するとセットされるので
// 符号館スペースと合わせ、最少2.5単位の文字間スペースが確保される
//++++++++++++++++++++++++++++++++++++++++++++++
void CharSP(void){
    if(reqCharSp){              // 文字間スペース要求があれば
        T2CONbits.TMR2ON = 0;   // TMR2 Off
        TMR2 = 0;               // ゼロにリセット
        PR2 = lenDOT;           // PRwoセット
        T2CONbits.TMR2ON = 1;   // TMR2 On
        PIR1bits.TMR2IF = 0;    // TMR2 フラグクリア
        reqCharSp = 0;          // 文字間スペース要求をクリア
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 符号速度を早める　 「+」SWが押されている時に呼ばれる
//++++++++++++++++++++++++++++++++++++++++++++++
void SpeedUp(void){
    beep();                 // ビープ音を鳴らす
    idxSpd--;               // 次の速度を選ぶ
    if(idxSpd < 0){         // 端まできたら更新せずに
        idxSpd = 0;
        beep();             // ビープ音を鳴らす
    }
    lenDOT = speed[idxSpd];     // 速度値を取得
    eeprom_write(1, idxSpd);    // EEPROM 1番地 書込み
    while(sw_P);                // SWが離れるのを待つ
    __delay_ms(10);
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 符号速度を遅くする　 「-」SWが押されている時に呼ばれる
//++++++++++++++++++++++++++++++++++++++++++++++
void SpeedDown(void){
    beep();                 // ビープ音を鳴らす
    idxSpd++;               // 次の速度を選ぶ
    if(idxSpd > 5){         // 端まできたら更新せずに
        idxSpd = 5;
        beep();             // ビープ音を鳴らす
    }
    lenDOT = speed[idxSpd];     // 速度値を取得
    eeprom_write(1, idxSpd);    // EEPROM 1番地 書込み
    while(sw_M);                // SWが離れるのを待つ
    __delay_ms(10);
}

//++++++++++++++++++++++++++++++++++++++++++++++
// Beep音を鳴らす　1kHz100ms - OFF50ms
//++++++++++++++++++++++++++++++++++++++++++++++
void beep(void){
    ToneOn();
        __delay_ms(100);
    ToneOff();
        __delay_ms(50);
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 縦振れ電鍵動作
//++++++++++++++++++++++++++++++++++++++++++++++
void StrightKey(){
    beep(); beep();             // beep 2回
    while(sw_P);                // SWが離れるのを待つ
    while(1){
        if(keyDASH) ToneOn();   // キーON で  Tone ON
        else ToneOff();         // キーOFF で Tone OFF
        if(sw_P) FreqUp();      // + SWなら速度を１段階早くする
        if(sw_M) FreqDown();    // - SWなら速度を１段階遅くする
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 「＋」SWを押す毎に100Hz単位で上げる
//++++++++++++++++++++++++++++++++++++++++++++++
void FreqUp(void){
    switch(TMR0H){
        case 125: TMR0H = 125; 
               beep(); beep(); break;   // 1kHzを維持
        case 139: TMR0H = 125; break;   // 1kHzに変更
        case 156: TMR0H = 139; break;   // 900Hzに変更
        case 179: TMR0H = 156; break;   // 800Hzに変更
        case 208: TMR0H = 179; break;   // 700Hzに変更
        default:  TMR0H = 208; break;   // 600Hzに変更
    }
    eeprom_write(0, TMR0H); // EEPROM 0番地 書込み
    ToneOn();               // 押している間 Tone On
    while(sw_P);            // SWが離れるのを待つ
    ToneOff();              // Tone Off
    __delay_ms(10);         // チャタリング防止
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 「ー」SWを押す毎に100Hz単位で下げる
//++++++++++++++++++++++++++++++++++++++++++++++
void FreqDown(void){
    switch(TMR0H){
        case 125: TMR0H = 139; break;   // 900Hzに変更
        case 139: TMR0H = 156; break;   // 800Hzに変更
        case 156: TMR0H = 179; break;   // 700Hzに変更
        case 179: TMR0H = 208; break;   // 600Hzに変更
        default:  TMR0H = 250; 
               beep(); beep(); break;   // 500Hzに変更
    }
    eeprom_write(0, TMR0H);
    ToneOn();
    while(sw_M);
    ToneOff();
    __delay_ms(10);
}

/** ++++++++ End of File ++++++++ */
